<?php
/**
 * @package   admintoolswp
 * @copyright Copyright (c)2017-2019 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU GPL version 3 or later
 */

use Akeeba\AdminTools\Admin\Model\OptimizeWaf;
use Akeeba\AdminTools\Library\Filesystem\File;
use Akeeba\AdminTools\Library\Input\Input;

/**
 * @package		admintoolswp
 * @copyright Copyright (c)2017-2019 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license		GNU GPL version 3 or later
 */

abstract class AdminToolsInstaller
{
	/**
	 * Obsolete files and folders to remove from the free version only. This is used when you move a feature from the
	 * free version of your extension to its paid version. If you don't have such a distinction you can ignore this.
	 *
	 * @var   array
	 */
	private static $removeFilesFree = array(
		'files'   => array(),
		'folders' => array(),
	);

	/**
	 * Obsolete files and folders to remove from both paid and free releases. This is used when you refactor code and
	 * some files inevitably become obsolete and need to be removed.
	 *
	 * @var   array
	 */
	private static $removeFilesAllVersions = array(
		'files'   => array(
			'app/plugins/waf/feature/secretword.php'
		),
		'folders' => array(),
	);

	private static $isPaid = false;

	public static function uninstall()
	{
		global $wpdb;

		// Clear the cron
		wp_clear_scheduled_hook('admintoolswp_cron');

		// Clear our hook to update functions. It seems WordPress invokes them after deleting the files, resulting in a
		// hidden fatal error during the uninstall process
		remove_filter ('pre_set_site_transient_update_plugins', array('AdminToolsWPUpdater', 'getupdates'), 10);

		//Remove the mu-plugins
		$mu_folder = ABSPATH.'wp-content/mu-plugins';

		if (defined('WPMU_PLUGIN_DIR') && WPMU_PLUGIN_DIR)
		{
			$mu_folder = WPMU_PLUGIN_DIR;
		}

		if (file_exists($mu_folder.'/admintoolswp.php'))
		{
			unlink($mu_folder.'/admintoolswp.php');
		}

		// Disable auto-prepend mode
		$input = new Input();
		$optimize = new OptimizeWaf($input);

		try
		{
			$optimize->disableAutoPrepend();
		}
		catch (Exception $e)
		{
			// Something bad happened. Let's cross our fingers and hope it won't break the site
		}

		// Uninstall tables
		$prefix = $wpdb->prefix;
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_acl");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_adminiplist");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_badwords");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_customperms");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_filescache");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_ipautoban");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_ipblock");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_ipautoban");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_log");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_redirects");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_scanalerts");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_scans");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_storage");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_scanalerts");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_wafexceptions");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_wafblacklists");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_profiles");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_waftemplates");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_ipautobanhistory");
		$wpdb->query("DROP TABLE IF EXISTS {$prefix}admintools_cookies");

		// Delete temporary data stored inside WP tables, too
		$wpdb->query("DELETE FROM `{$prefix}options` WHERE `option_name` = 'admintoolswp_plugin_dir'");
		$wpdb->query("DELETE FROM `{$prefix}usermeta` WHERE `meta_key` = 'admintoolswp_messages'");
	}

	/**
	 * This function is called everytime we install, update or activate our plugin
	 */
	public static function installOrUpdate()
	{
		// Require WordPress 3.1 or later
		if (version_compare(get_bloginfo('version'), '3.1', 'lt'))
		{
			deactivate_plugins('admintoolswp.php');

			return;
		}

		// Check that we have the correct PHP version
		$minPHP = defined('ADMINTOOLSWP_MIN_PHP') ? ADMINTOOLSWP_MIN_PHP : '5.4.0';

		if (version_compare(PHP_VERSION, $minPHP, 'lt'))
		{
			deactivate_plugins('admintoolswp.php');

			return;
		}

		unset($minPHP);


		// Clear opcache before starting the update process
		self::clearOpcodeCaches();

		// Register cron hook and our custom interval
		if (!wp_next_scheduled('admintoolswp_cron'))
		{
			wp_schedule_event(time(), 'five_min', 'admintoolswp_cron');
		}

		// First of all save plugin path on filesystem, for installation on different folder names
		self::savePluginPath();

		$admintools_dir = get_option('admintoolswp_plugin_dir');
		$engine_path 	= WP_PLUGIN_DIR . '/' . $admintools_dir . '/app/engine';
		self::$isPaid = is_dir($engine_path);

		// Create database tables
		self::createTables();

		// Install default data
		self::installData();

		// Install (copy) the mu plugin inside the mu-folder
		self::installPlugin();

		// Which files should I remove?
		if (self::$isPaid)
		{
			// This is the paid version, only remove the removeFilesAllVersions files
			$removeFiles = self::$removeFilesAllVersions;
		}
		else
		{
			// This is the free version, remove the removeFilesAllVersions and removeFilesFree files
			$removeFiles['files']   = array_merge(self::$removeFilesAllVersions['files'], self::$removeFilesFree['files']);
			$removeFiles['folders'] = array_merge(self::$removeFilesAllVersions['folders'], self::$removeFilesFree['folders']);
		}

		// Remove obsolete files and folders
		self::removeFilesAndFolders($removeFiles);

		// Clear opcache AFTER we did our job
		self::clearOpcodeCaches();
	}

	/**
	 * This function will inspect if the given file (usually one not included in the plugin folder) is up to date.
	 * It will extract the version from the passed constant and check vs the current one.
	 *
	 * @param	string	$file		File to check
	 * @param	string	$constant	Constant that should be extracted
	 *
	 * @return bool
	 */
	public static function fileUpToDate($file, $constant)
	{
		// No file? This means that the user disabled it or not enabled the feature. Let's stop here
		if (!file_exists($file))
		{
			return true;
		}

		$contents = file_get_contents($file);
		$lines	  = explode("\n", $contents);

		foreach ($lines as $line)
		{
			if (strpos($line, $constant) === false)
			{
				continue;
			}

			$version = explode(',', $line);
			$version = trim($version[1], ' );');
			$version = str_replace('\'', '', $version);
			$version = str_replace('"', '', $version);

			return version_compare(ADMINTOOLSWP_VERSION, $version, 'eq');
		}

		return true;
	}

	public static function updateAutoPrependFile()
	{
		$waf_file = ABSPATH.'admintools-waf.php';

		// If the file exists, let's update it, no matter what
		if (file_exists($waf_file))
		{
			$input		 = new Input();
			$optimizeWaf = new OptimizeWaf($input);

			$optimizeWaf->createAutoPrependFile($waf_file);
		}
	}

	/**
	 * Clear PHP opcode caches
	 *
	 * @return  void
	 */
	public static function clearOpcodeCaches()
	{
		// Always reset the OPcache if it's enabled. Otherwise there's a good chance the server will not know we are
		// replacing .php scripts. This is a major concern since PHP 5.5 included and enabled OPcache by default.
		if (function_exists('opcache_reset'))
		{
			opcache_reset();
		}
		// Also do that for APC cache
		elseif (function_exists('apc_clear_cache'))
		{
			@apc_clear_cache();
		}
	}

	/**
	 * Removes obsolete files and folders
	 *
	 * @param   array $removeList The files and directories to remove
	 */
	private static function removeFilesAndFolders($removeList)
	{
		$plugin_dir  = get_option('admintoolswp_plugin_dir');
		$plugin_path = WP_PLUGIN_DIR . '/' . $plugin_dir;

		// Remove files
		if (isset($removeList['files']) && !empty($removeList['files']))
		{
			foreach ($removeList['files'] as $file)
			{
				$f = $plugin_path . $file;

				if (!is_file($f))
				{
					continue;
				}

				// TODO Use WordPress Filesystem API
				@unlink($f);
			}
		}

		// Remove folders
		if (isset($removeList['folders']) && !empty($removeList['folders']))
		{
			// TODO At the moment we use our own adapter, in the future we'll have to use WordPress FS API
			$filesystem = new File();

			foreach ($removeList['folders'] as $folder)
			{
				$f = $plugin_path . $folder;

				if (!is_dir($f))
				{
					continue;
				}

				$filesystem->rmdir($f, true);
			}
		}
	}

	public static function savePluginPath()
	{
		$baseUrlParts = explode('/', plugins_url('', __FILE__));
		array_pop($baseUrlParts);
		$pluginDir 	  = end($baseUrlParts);

		// Always update the plugin path, no matter what
		update_option('admintoolswp_plugin_dir', $pluginDir);
	}

	public static function installPlugin()
	{
		// Copy the mu-plugins in the correct folder
		$mu_folder = ABSPATH.'wp-content/mu-plugins';

		if (defined('WPMU_PLUGIN_DIR') && WPMU_PLUGIN_DIR)
		{
			$mu_folder = WPMU_PLUGIN_DIR;
		}

		if (!is_dir($mu_folder))
		{
			mkdir($mu_folder, 0755, true);
		}

		// If the file already exists and it's a link, do not copy over (local development)
		if (is_link($mu_folder.'/admintoolswp.php'))
		{
			return;
		}

		$result = copy(ADMINTOOLSWP_PATH . '/app/plugins/mu-plugins/admintoolswp.php', $mu_folder.'/admintoolswp.php');

		// If something went bad, stop here since it won't work as intended
		if (!$result)
		{
			throw new \RuntimeException(sprintf('Can not copy mu-plugins into the folder %s', $mu_folder));
		}
	}

	public static function createTables()
	{
		global $wpdb;

		require_once ABSPATH . 'wp-admin/includes/upgrade.php';

		$prefix          = $wpdb->prefix;
		$charset_collate = $wpdb->get_charset_collate();

		/*
		 * Ok, here's the deal: WP documentation claims that we have to avoid quoting column names with backticks (`),
		 * but we are using fields with a reserved name (ie `key`). However I tried to run the queries with the backtick and it works fine.
		 * The only downside is for the columns named "key": WP thinks he has to create a new index and throws an error.
		 * Finally it seems that indexes with the prefix in the name are causing errors, so I'll remove them
		 */
		$queries[] = "CREATE TABLE {$prefix}admintools_acl (
							user_id bigint(20) unsigned NOT NULL,
							permissions mediumtext,
							PRIMARY KEY  (user_id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_adminiplist (
							id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
							ip varchar(255) DEFAULT NULL,
							description varchar(255) DEFAULT NULL,
							PRIMARY KEY  (id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_badwords (
							id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
							word varchar(255) DEFAULT NULL,
							PRIMARY KEY  (id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_customperms (
							id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
							path varchar(100) NOT NULL,
							perms varchar(4) DEFAULT '0644',
							PRIMARY KEY  (id),
							KEY admintools_customperms_path (path)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_filescache (
							admintools_filescache_id bigint(20) NOT NULL AUTO_INCREMENT,
							path varchar(2048) NOT NULL,
							filedate int(11) NOT NULL DEFAULT '0',
							filesize int(11) NOT NULL DEFAULT '0',
							data blob,
							checksum varchar(32) NOT NULL DEFAULT '',
							PRIMARY KEY  (admintools_filescache_id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_ipautoban (
							ip varchar(255) NOT NULL,
							reason varchar(255) DEFAULT 'other',
							until datetime DEFAULT NULL,
							PRIMARY KEY  (ip(100))
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_ipblock (
							id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
							ip varchar(255) DEFAULT NULL,
							description varchar(255) DEFAULT NULL,
							PRIMARY KEY  (id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_log (
							id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
							logdate datetime NOT NULL,
							ip varchar(40) DEFAULT NULL,
							url varchar(255) DEFAULT NULL,
							reason enum('other', 'admindir', 'awayschedule', 'adminpw','ipwl','ipbl','sqlishield','antispam','tpone','tmpl','template','muashield','csrfshield','badbehaviour','geoblocking','rfishield','dfishield','uploadshield','xssshield','httpbl','loginfailure','securitycode', 'sessionshield', 'external', 'nonewadmins', 'nonewfrontendadmin', 'configmonitor', 'phpshield') DEFAULT 'other',
							extradata mediumtext,
							PRIMARY KEY  (id),
							KEY admintools_log_logdate_reason (logdate, reason)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_redirects (
							id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
							source varchar(255) DEFAULT NULL,
							dest varchar(255) DEFAULT NULL,
							ordering bigint(20) NOT NULL DEFAULT '0',
							published tinyint(1) NOT NULL DEFAULT '1',
							keepurlparams tinyint(1) NOT NULL DEFAULT '1',
							PRIMARY KEY  (id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_scanalerts (
							admintools_scanalert_id bigint(20) NOT NULL AUTO_INCREMENT,
							path varchar(2048) NOT NULL,
							scan_id bigint(20) NOT NULL DEFAULT '0',
							diff mediumtext,
							threat_score int(11) NOT NULL DEFAULT '0',
							acknowledged tinyint(4) NOT NULL DEFAULT '0',
							PRIMARY KEY  (admintools_scanalert_id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_scans (
							id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
							description varchar(255) NOT NULL,
							comment longtext,
							backupstart timestamp NOT NULL DEFAULT '0000-00-00 00:00:00',
							backupend timestamp NOT NULL DEFAULT '0000-00-00 00:00:00',
							status enum('run','fail','complete') NOT NULL DEFAULT 'run',
							origin varchar(30) NOT NULL DEFAULT 'backend',
							type varchar(30) NOT NULL DEFAULT 'full',
							profile_id bigint(20) NOT NULL DEFAULT '1',
							archivename longtext,
							absolute_path longtext,
							multipart int(11) NOT NULL DEFAULT '0',
							tag varchar(255) DEFAULT NULL,
							backupid varchar(255) DEFAULT NULL,
							filesexist tinyint(3) NOT NULL DEFAULT '1',
							remote_filename varchar(1000) DEFAULT NULL,
							total_size bigint(20) NOT NULL DEFAULT '0',
							PRIMARY KEY  (id),
							KEY idx_fullstatus (filesexist,status),
							KEY idx_stale (status,origin)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_storage (
							`at_key` varchar(255) NOT NULL,
							`at_value` longtext NOT NULL,
							PRIMARY KEY  (`at_key`(100))
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_wafexceptions (
						  `id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
						  `at_url` varchar(255) NOT NULL,
						  `descr` varchar(255) NOT NULL,
						  `at_type` enum('exact','regex') NOT NULL,
						  `published` tinyint(1) NOT NULL,
						  PRIMARY KEY (`id`)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_wafblacklists (
							id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
							`option` varchar(255) NOT NULL,
							`view` varchar(255) NOT NULL,
							task varchar(255) NOT NULL,
							query varchar(255) NOT NULL,
							query_type varchar(1) NOT NULL,
							query_content varchar(255) NOT NULL,
							verb varchar(6) NOT NULL,
							enabled TINYINT(3) NOT NULL DEFAULT 1,
							PRIMARY KEY  (id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_profiles (
							id int(10) unsigned NOT NULL AUTO_INCREMENT,
							description varchar(255) NOT NULL,
							configuration longtext,
							filters longtext,
							PRIMARY KEY  (id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_waftemplates (
						  admintools_waftemplate_id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
						  reason varchar(255) NOT NULL,
						  language varchar(10) NOT NULL DEFAULT '*',
						  subject varchar(255) NOT NULL,
						  template text NOT NULL,
						  enabled TINYINT(3) NOT NULL DEFAULT 1,
						  email_num tinyint(3) unsigned NOT NULL,
						  email_numfreq tinyint(3) unsigned NOT NULL,
						  email_freq enum('','second','minute','hour','day') NOT NULL DEFAULT '',
						  created_on datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
						  created_by bigint(20) NOT NULL DEFAULT '0',
						  modified_on datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
						  modified_by bigint(20) NOT NULL DEFAULT '0',
						  PRIMARY KEY  (admintools_waftemplate_id),
						  UNIQUE KEY admintools_waftemplate_keylang (reason(100), language)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_ipautobanhistory (
						  id bigint(20) unsigned NOT NULL AUTO_INCREMENT,
						  ip varchar(255) NOT NULL,
						  reason varchar(255) DEFAULT 'other',
						  until datetime DEFAULT NULL,
						  PRIMARY KEY  (id)
						) $charset_collate;";

		$queries[] = "CREATE TABLE {$prefix}admintools_cookies (
						  series varchar(255) NOT NULL,
						  client_hash varchar(255) NOT NULL,
						  valid_to datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
						  PRIMARY KEY  (series(100))
						) $charset_collate;";

		dbDelta($queries);

		// Manually run this query, since it's a table used by Akeeba for Wordpress, too
		$query = "CREATE TABLE IF NOT EXISTS {$prefix}akeeba_common (
							`key` varchar(255) NOT NULL,
							`value` longtext NOT NULL,
							PRIMARY KEY (`key`(100))
						) $charset_collate;";

		$wpdb->query($query);
	}

	public static function installData()
	{
		global $wpdb;

		$prefix = $wpdb->prefix;

		// Check for default PHP Scanner profile
		$sql   = "SELECT COUNT(*) FROM `{$prefix}admintools_profiles` WHERE `id` = 1";
		$count = $wpdb->get_var($sql);

		if (!$count)
		{
			$wpdb->insert(
				$prefix.'admintools_profiles',
				array(
					'id'            => '1',
					'description'   => 'Default PHP Change Scanner Profile',
					'configuration' => '',
					'filters'       => ''
				)
			);
		}

		$sql   = "SELECT COUNT(*) FROM `{$prefix}admintools_waftemplates`";
		$count = $wpdb->get_var($sql);

		if (!$count)
		{
			$wpdb->insert(
				$prefix.'admintools_waftemplates',
				array(
					'reason' => 'all',
					'language' => '*',
					'subject' => 'Security exception on [SITENAME]',
					'template' => "<div style=\"background-color: #e0e0e0; padding: 10px 20px;\">\r\n<div style=\"background-color: #f9f9f9; border-radius: 10px; padding: 5px 10px;\">\r\n<p>Hello,</p>\r\n<p>We would like to notify you that a security exception was detected on your site, [SITENAME], with the following details:</p>\r\n<p>IP Address: [IP] (IP Lookup: [LOOKUP])<br />Reason: [REASON]</p>\r\n<p>If this kind of security exception repeats itself, please log in to your site's back-end and add this IP address to your Admin Tools's Web Application Firewall feature in order to completely block the misbehaving user.</p>\r\n<p>Best regards,</p>\r\n<p>The [SITENAME] team</p>\r\n</div>\r\n<p style=\"font-size: x-small; color: #667;\">You are receiving this automatic email message because you are an administrator in <em>[SITENAME]</em>. <span style=\"line-height: 1.3em;\">Do not reply to this email, it's sent from an unmonitored email address .</span ></p > \r\n</div >",
					'enabled' => '1',
					'email_num' => '5',
					'email_numfreq' => '1',
					'email_freq' => 'hour'
				)
			);
		}

		$sql   = "SELECT COUNT(*) FROM `{$prefix}admintools_waftemplates` WHERE `reason` = 'adminloginfail'";
		$count = $wpdb->get_var($sql);

		if (!$count)
		{
			$wpdb->insert(
				$prefix.'admintools_waftemplates',
				array(
					'reason' => 'adminloginfail',
					'language' => '*',
					'subject' => 'Failed administrator login for user [USER] on [SITENAME]',
					'template' => "<div style=\"background-color: #e0e0e0; padding: 10px 20px;\">\r\n<div style=\"background-color: #f9f9f9; border-radius: 10px; padding: 5px 10px;\">\r\n<p>Hello,</p>\r\n<p>We would like to notify you that user <strong>[USER]</strong> did <strong>not</strong> log in successfully to the administrator back-end area of your site, [SITENAME]. <span style=\"font-size: 12px; line-height: 15px;\">Further information:</span></p>\r\n<p style=\"font-size: 12px; line-height: 15px;\">Username: [USER]<br />IP address: [IP] ([LOOKUP])<br />Country*: [COUNTRY]<br />Continent*: [CONTINENT]<br />Browser User Agent string: [UA]</p>\r\n<p style=\"font-size: 12px; line-height: 15px;\">* Country and continent information availability and accuracy depend on the GeoIP database installed on your site.</p>\r\n<p>If this expected you do not need to do anything about it. If you suspect a hacking attempt, please log in to your site's back-end immediately and activate Admin Tools' Emergency Off-Line mode at once.</p>\r\n<p>Best regards,</p>\r\n<p>The [SITENAME] team</p>\r\n</div>\r\n<p style=\"font-size: x-small; color: #667;\">You are receiving this automatic email message because you are an administrator in <em>[SITENAME]</em>. <span style=\"line-height: 1.3em;\">Do not reply to this email, it's sent from an unmonitored email address.</span></p>",
					'enabled' => '1',
					'email_num' => '5',
					'email_numfreq' => '1',
					'email_freq' => 'hour'
				)
			);

			$wpdb->insert(
				$prefix.'admintools_waftemplates',
				array(
					'reason' => 'adminloginsuccess',
					'language' => '*',
					'subject' => 'User [USER] logged in on [SITENAME] administrator area',
					'template' => "<div style=\"background-color: #e0e0e0; padding: 10px 20px;\">\r\n<div style=\"background-color: #f9f9f9; border-radius: 10px; padding: 5px 10px;\">\r\n<p style=\"font-size: 12px; line-height: 15px;\">Hello,</p>\r\n<p>We would like to notify you that user [USER] has just logged in to the administrator back-end area of your site, [SITENAME]. Further information:</p>\r\n<p>Username: [USER]<br />IP address: [IP] ([LOOKUP])<br />Country*: [COUNTRY]<br />Continent*: [CONTINENT]<br />Browser User Agent string: [UA]</p>\r\n<p>* Country and continent information availability and accuracy depend on the GeoIP database installed on your site.</p>\r\n<p>If this expected you do not need to do anything about it. If you suspect a hacking attempt, please log in to your site's back-end immediately and activate Admin Tools' Emergency Off-Line mode at once.</p>\r\n<p style=\"font-size: 12px; line-height: 15px;\">Best regards,</p>\r\n<p style=\"font-size: 12px; line-height: 15px;\">The [SITENAME] team</p>\r\n</div>\r\n<p style=\"font-size: x-small; color: #667;\">You are receiving this automatic email message because you are an administrator in <em>[SITENAME]</em>. <span style=\"line-height: 1.3em;\">Do not reply to this email, it's sent from an unmonitored email address.</span></p>",
					'enabled' => '1',
					'email_num' => '5',
					'email_numfreq' => '1',
					'email_freq' => 'hour'
				)
			);

			$wpdb->insert(
				$prefix.'admintools_waftemplates',
				array(
					'reason' => 'ipautoban',
					'language' => '*',
					'subject' => 'Automatic IP blocking notification for [IP] on [SITENAME]',
					'template' => "<div style=\"background-color: #e0e0e0; padding: 10px 20px;\">\r\n<div style=\"background-color: #f9f9f9; border-radius: 10px; padding: 5px 10px;\">\r\n<p style=\"font-size: 12px; line-height: 15px;\">Hello,</p>\r\n<p>We would like to notify you that the IP address [IP] is now blocked from accessing your site, [SITENAME]<span style=\"font-size: 12px; line-height: 15px;\">, with the following details:</span></p>\r\n<p style=\"font-size: 12px; line-height: 15px;\">IP Address: [IP] (IP Lookup: [LOOKUP])<br />Reason: [REASON]<br />Banned until: [UNTIL]</p>\r\n<p>If this is your own IP address and you can no longer access your site please <a href=\"http://akee.ba/lockedout\">follow our instructions</a> to temporarily disable Admin Tools' Web Application Firewall and clear the automatic IP ban.</p>\r\n<p style=\"font-size: 12px; line-height: 15px;\">Best regards,</p>\r\n<p style=\"font-size: 12px; line-height: 15px;\">The [SITENAME] team</p>\r\n</div>\r\n<p style=\"font-size: x-small; color: #667;\">You are receiving this automatic email message because you are an administrator in <em>[SITENAME]</em>. <span style=\"line-height: 1.3em;\">Do not reply to this email, it's sent from an unmonitored email address.</span></p>",
					'enabled' => '1',
					'email_num' => '5',
					'email_numfreq' => '1',
					'email_freq' => 'hour'
				)
			);
		}
	}
}
